import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {State} from "../../project/game-model/gamesystems/states/State";
import {Gamesystem} from "../../project/game-model/gamesystems/Gamesystem";
import { Transition } from '../../project/game-model/gamesystems/transitions/Transition';
import {ScriptAccount} from "../../project/game-model/scriptAccounts/ScriptAccount";
import {SimpleGamesystem} from "../../project/game-model/gamesystems/SimpleGamesystem";
import {ProductGamesystem} from "../../project/game-model/gamesystems/ProductGamesystem";
import {SimpleTemplateGamesystem} from "../../project/game-model/gamesystems/SimpleTemplateGamesystem";
import {Character} from "../../project/game-model/characters/Character";

@Component({
  selector: 'app-gamesystem-editor',
  templateUrl: './gamesystem-editor.component.html',
  styleUrl: './gamesystem-editor.component.scss'
})
export class GamesystemEditorComponent implements OnInit{

  @Input() gamesystem: Gamesystem<State<any>, Transition<any>> | undefined
  @Input() scriptAccounts: ScriptAccount[] = [];
  @Input() templateReference: Character | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEmitter = new EventEmitter<SimpleGamesystem>();

  ngOnInit() {
    console.log("GamesystemEditor: ", this.scriptAccounts.length)
  }

  isSimpleGamesystem() {
    return this.gamesystem instanceof SimpleGamesystem || this.gamesystem instanceof SimpleTemplateGamesystem;
  }

  convertGamesystemToSimpleGamesystem() {
    if(!(this.gamesystem instanceof ProductGamesystem)) {
      return this.gamesystem as SimpleGamesystem;
    }
  }

  convertGamesystemToProductGamesystem() {
    if(this.gamesystem instanceof ProductGamesystem) {
      return this.gamesystem as ProductGamesystem;
    }
  }

  onOpenGamesystemEditor(gamesystem: SimpleGamesystem) {
    this.openGamesystemEmitter.emit(gamesystem);
  }
}
