import {StoreComponent} from "../../../../../app/storage/StoreComponent";
import {Character} from "../../game-model/characters/Character";
import {SimpleTemplateGamesystem} from "../../game-model/gamesystems/SimpleTemplateGamesystem";
import {ScriptAccount} from "../../game-model/scriptAccounts/ScriptAccount";
import {ScriptAccountParser} from "../ScriptAccountParser";
import {ScriptAccountConditionParser} from "../gamesystemParser/ScriptAccountConditionParser";


export class CharacterParser {

  characterSpecificGamesystems: SimpleTemplateGamesystem<Character>[]
  scriptAccountConditionParser: ScriptAccountConditionParser

  constructor(characterSpecificGamesystems: SimpleTemplateGamesystem<Character>[], scriptAccounts: ScriptAccount[]) {
    this.characterSpecificGamesystems = characterSpecificGamesystems;
    this.scriptAccountConditionParser = new ScriptAccountConditionParser(scriptAccounts)
  }

  public parseCharacters(characters: StoreComponent[]): Character[] {
    const loadedCharacters: Character[] = []
    characters.forEach(character => loadedCharacters.push(this.parseSingleCharacter(JSON.parse(character.jsonString))))
    return loadedCharacters;
  }

  private parseSingleCharacter(characterData: any): Character {
    const character = new Character(characterData.componentName, characterData.componentDescription);
    character.characterSpecificGamesystems = this.parseCharacterSpecificGamesystems(character, characterData.characterSpecificGamesystems);
    return character;
  }

  private parseCharacterSpecificGamesystems(character: Character, characterSpecificGamesystems: any): SimpleTemplateGamesystem<Character>[] {
    const result: SimpleTemplateGamesystem<Character>[] = []
    for(let i=0; i<characterSpecificGamesystems.length; i++) {
      const characterSpecificGamesystem = characterSpecificGamesystems[i];
      result.push(this.parseSingleCharacterSpecificGamesystem(character, characterSpecificGamesystem)!)
    }
    return result;
  }

  private parseSingleCharacterSpecificGamesystem(character: Character, characterSpecificGamesystem: any): SimpleTemplateGamesystem<Character> | undefined{
    const referencedGamesystem = this.findCharacterSpecificGamesystem(characterSpecificGamesystem.componentName)

    if(referencedGamesystem != undefined) {
      for(let i=0; i<characterSpecificGamesystem.states.length; i++) {
        const stateReference = characterSpecificGamesystem.states[i];
        const state = this.findReferencedState(referencedGamesystem, stateReference.stateLabel)!

        const conditions = this.scriptAccountConditionParser.parseStoredConditions(stateReference.conditionMap);

        state.conditionMap.set(character, conditions)
      }
    }

    return referencedGamesystem;
  }
  private findCharacterSpecificGamesystem(componentName: string): SimpleTemplateGamesystem<Character> | undefined{
    return this.characterSpecificGamesystems.find(gamesystem => gamesystem.componentName === componentName)
  }

  private findReferencedState(gamesystem: SimpleTemplateGamesystem<Character>, stateLabel: string) {
    return gamesystem.states.find(state => state.stateLabel === stateLabel);
  }
}
